@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; register specs
 
@@ -96,9 +96,9 @@
   (eval-when (:compile-toplevel :load-toplevel :execute)
     (defparameter *register-arg-names* '(rdx rdi rsi)))
   (defregset    *register-arg-offsets* rdx rdi rsi)
-  #!-win32
+  #-win32
   (defregset    *c-call-register-arg-offsets* rdi rsi rdx rcx r8 r9)
-  #!+win32
+  #+win32
   (defregset    *c-call-register-arg-offsets* rcx rdx r8 r9))
 
 ;;;; SB definitions
@@ -132,10 +132,12 @@
   (fp-complex-single-immediate immediate-constant)
   (fp-complex-double-immediate immediate-constant)
 
-  #!+sb-simd-pack (int-sse-immediate immediate-constant)
-  #!+sb-simd-pack (double-sse-immediate immediate-constant)
-  #!+sb-simd-pack (single-sse-immediate immediate-constant)
-
+  #+sb-simd-pack (int-sse-immediate immediate-constant)
+  #+sb-simd-pack (double-sse-immediate immediate-constant)
+  #+sb-simd-pack (single-sse-immediate immediate-constant)
+  #+sb-simd-pack-256 (int-avx2-immediate immediate-constant)
+  #+sb-simd-pack-256 (double-avx2-immediate immediate-constant)
+  #+sb-simd-pack-256 (single-avx2-immediate immediate-constant)
   (immediate immediate-constant)
 
   ;;
@@ -154,12 +156,18 @@
   (double-stack stack)
   (complex-single-stack stack)  ; complex-single-floats
   (complex-double-stack stack :element-size 2)  ; complex-double-floats
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (int-sse-stack stack :element-size 2)
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (double-sse-stack stack :element-size 2)
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (single-sse-stack stack :element-size 2)
+  #+sb-simd-pack-256
+  (int-avx2-stack stack :element-size 4)
+  #+sb-simd-pack-256
+  (double-avx2-stack stack :element-size 4)
+  #+sb-simd-pack-256
+  (single-avx2-stack stack :element-size 4)
 
   ;;
   ;; magic SCs
@@ -244,28 +252,49 @@
                       :alternate-scs (complex-double-stack))
 
   ;; temporary only
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (sse-reg float-registers
            :locations #.*float-regs*)
   ;; regular values
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (int-sse-reg float-registers
                :locations #.*float-regs*
                :constant-scs (int-sse-immediate)
                :save-p t
                :alternate-scs (int-sse-stack))
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (double-sse-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (double-sse-immediate)
                   :save-p t
                   :alternate-scs (double-sse-stack))
-  #!+sb-simd-pack
+  #+sb-simd-pack
   (single-sse-reg float-registers
                   :locations #.*float-regs*
                   :constant-scs (single-sse-immediate)
                   :save-p t
                   :alternate-scs (single-sse-stack))
+  #+avx2
+  (avx2-reg float-registers
+   :locations #.*float-regs*)
+  #+sb-simd-pack-256
+  (int-avx2-reg float-registers
+               :locations #.*float-regs*
+               :constant-scs (int-avx2-immediate)
+               :save-p t
+               :alternate-scs (int-avx2-stack))
+  #+sb-simd-pack-256
+  (double-avx2-reg float-registers
+                  :locations #.*float-regs*
+                  :constant-scs (double-avx2-immediate)
+                  :save-p t
+                  :alternate-scs (double-avx2-stack))
+  #+sb-simd-pack-256
+  (single-avx2-reg float-registers
+                  :locations #.*float-regs*
+                  :constant-scs (single-avx2-immediate)
+                  :save-p t
+                  :alternate-scs (single-avx2-stack))
 
   (catch-block stack :element-size catch-block-size)
   (unwind-block stack :element-size unwind-block-size)))
@@ -282,20 +311,25 @@
 (defparameter *double-sc-names* '(double-reg double-stack))
 (defparameter *complex-sc-names* '(complex-single-reg complex-single-stack
                                    complex-double-reg complex-double-stack))
-#!+sb-simd-pack
-;;; FIXME: there is no SSE-STACK storage class
+#+sb-simd-pack
 (defparameter *oword-sc-names* '(sse-reg int-sse-reg single-sse-reg double-sse-reg
-                                 sse-stack int-sse-stack single-sse-stack double-sse-stack))
+                                 int-sse-stack single-sse-stack double-sse-stack))
+#+sb-simd-pack-256
+(defparameter *hword-sc-names* '(avx2-reg int-avx2-reg single-avx2-reg double-avx2-reg
+                                   int-avx2-stack single-avx2-stack double-avx2-stack))
 ) ; EVAL-WHEN
 (!define-storage-classes
   . #.(mapcar (lambda (class-spec)
                 (let ((size
-                       (case (car class-spec)
-                         (#.*oword-sc-names*   :oword)
-                         (#.*qword-sc-names*   :qword)
-                         (#.*float-sc-names*   :float)
-                         (#.*double-sc-names*  :double)
-                         (#.*complex-sc-names* :complex))))
+                        (case (car class-spec)
+                          #+sb-simd-pack
+                          (#.*oword-sc-names*   :oword)
+                          #+sb-simd-pack-256
+                          (#.*hword-sc-names*   :hword)
+                          (#.*qword-sc-names*   :qword)
+                          (#.*float-sc-names*   :float)
+                          (#.*double-sc-names*  :double)
+                          (#.*complex-sc-names* :complex))))
                   (append class-spec (if size (list :operand-size size)))))
               *storage-class-defs*))
 
@@ -338,6 +372,10 @@
 (defun xmm-tn-p (thing)
   (and (tn-p thing)
        (eq (sb-name (sc-sb (tn-sc thing))) 'float-registers)))
+;;; Return true if THING is on the stack (in whatever storage class).
+(defun stack-tn-p (thing)
+  (and (tn-p thing)
+       (eq (sb-name (sc-sb (tn-sc thing))) 'stack)))
 
 ;; A register that's never used by the code generator, and can therefore
 ;; be used as an assembly temporary in cases where a VOP :TEMPORARY can't
@@ -363,15 +401,15 @@
 ;;; the appropriate SC number, otherwise return NIL.
 (defun immediate-constant-sc (value)
   (typecase value
-    ((or (integer #.sb!xc:most-negative-fixnum #.sb!xc:most-positive-fixnum)
+    ((or (integer #.sb-xc:most-negative-fixnum #.sb-xc:most-positive-fixnum)
          character)
      immediate-sc-number)
     (symbol ; Symbols in static and immobile space are immediate
-     (when (or ;; With #!+immobile-symbols, all symbols are in immobile-space.
+     (when (or ;; With #+immobile-symbols, all symbols are in immobile-space.
                ;; And the cross-compiler always uses immobile-space if enabled.
-               #!+(or immobile-symbols (and immobile-space (host-feature sb-xc-host))) t
+               #+(or immobile-symbols (and immobile-space sb-xc-host)) t
 
-               ;; Otherwise, if #!-immobile-symbols, and the symbol was present
+               ;; Otherwise, if #-immobile-symbols, and the symbol was present
                ;; in the initial core image as indicated by the symbol header, then
                ;; it's in immobile-space. There is a way in which the bit can be wrong,
                ;; but it's highly unlikely - the symbol would have to be uninterned from
@@ -379,34 +417,40 @@
                ;; initial package. All without breaking anything. Hence, unlikely.
                ;; Also note that if compiling to memory, the symbol's current address
                ;; is used to determine whether it's immediate.
-               #!+(and (not (host-feature sb-xc-host)) immobile-space (not immobile-symbols))
+               #+(and (not sb-xc-host) immobile-space (not immobile-symbols))
                (or (logbitp +initial-core-symbol-bit+ (get-header-data value))
-                   (and (sb!c::core-object-p sb!c::*compile-object*)
+                   (and (sb-c::core-object-p sb-c::*compile-object*)
                         (immobile-space-obj-p value)))
 
                (static-symbol-p value))
        immediate-sc-number))
-    #!+immobile-space
+    #+immobile-space
     (layout
        immediate-sc-number)
     (single-float
-       (if (eql value 0f0) fp-single-zero-sc-number fp-single-immediate-sc-number))
+       (if (eql value $0f0) fp-single-zero-sc-number fp-single-immediate-sc-number))
     (double-float
-       (if (eql value 0d0) fp-double-zero-sc-number fp-double-immediate-sc-number))
+       (if (eql value $0d0) fp-double-zero-sc-number fp-double-immediate-sc-number))
     ((complex single-float)
-       (if (eql value #c(0f0 0f0))
+       (if (eql value (complex $0f0 $0f0))
             fp-complex-single-zero-sc-number
             fp-complex-single-immediate-sc-number))
     ((complex double-float)
-       (if (eql value #c(0d0 0d0))
+       (if (eql value (complex $0d0 $0d0))
             fp-complex-double-zero-sc-number
             fp-complex-double-immediate-sc-number))
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack (not sb-xc-host))
     ((simd-pack double-float) double-sse-immediate-sc-number)
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
+    #+(and sb-simd-pack (not sb-xc-host))
     ((simd-pack single-float) single-sse-immediate-sc-number)
-    #!+(and sb-simd-pack (not (host-feature sb-xc-host)))
-    (simd-pack int-sse-immediate-sc-number)))
+    #+(and sb-simd-pack (not sb-xc-host))
+    (simd-pack int-sse-immediate-sc-number)
+    #+(and sb-simd-pack-256 (not sb-xc-host))
+    ((simd-pack-256 double-float) double-avx2-immediate-sc-number)
+    #+(and sb-simd-pack-256 (not sb-xc-host))
+    ((simd-pack-256 single-float) single-avx2-immediate-sc-number)
+    #+(and sb-simd-pack-256 (not sb-xc-host))
+    (simd-pack-256 int-avx2-immediate-sc-number)))
 
 (defun boxed-immediate-sc-p (sc)
   (eql sc immediate-sc-number))
@@ -418,8 +462,8 @@
           (integer  (if tag (fixnumize val) val))
           (symbol   (if (static-symbol-p val)
                         (+ nil-value (static-symbol-offset val))
-                        (make-fixup val :immobile-object)))
-          #!+immobile-space
+                        (make-fixup val :immobile-symbol)))
+          #+immobile-space
           (layout
            (make-fixup val :layout))
           (character (if tag
@@ -469,15 +513,16 @@
       (immediate-constant "Immed")
       (noise (symbol-name (sc-name sc))))))
 
+(defconstant nargs-offset rcx-offset)
 (defconstant cfp-offset rbp-offset) ; pfw - needed by stuff in /code
 
 (defun combination-implementation-style (node)
-  (declare (type sb!c::combination node))
+  (declare (type sb-c::combination node))
   (flet ((valid-funtype (args result)
-           (sb!c::valid-fun-use node
-                                (sb!c::specifier-type
+           (sb-c::valid-fun-use node
+                                (sb-c::specifier-type
                                  `(function ,args ,result)))))
-    (case (sb!c::combination-fun-source-name node)
+    (case (sb-c::combination-fun-source-name node)
       (logtest
        (cond
          ((or (valid-funtype '(fixnum fixnum) '*)
@@ -489,16 +534,12 @@
          (t
           (values :default nil))))
       (logbitp
-       (cond
-         ((or (and (valid-funtype '#.`((integer 0 ,(- 63 n-fixnum-tag-bits))
-                                       fixnum) '*)
-                   (sb!c::constant-lvar-p
-                    (first (sb!c::basic-combination-args node))))
-              (valid-funtype '((integer 0 63) (signed-byte 64)) '*)
-              (valid-funtype '((integer 0 63) (unsigned-byte 64)) '*))
-          (values :transform '(lambda (index integer)
-                               (%logbitp integer index))))
-         (t
-          (values :default nil))))
+       (if (or
+            (valid-funtype '((mod 64) word) '*)
+            (valid-funtype '((mod 64) signed-word) '*))
+           (values :transform '(lambda (index integer) (%logbitp index integer)))
+           (values :default nil)))
       (t
        (values :default nil)))))
+
+(defparameter *register-names* +qword-register-names+)
