@@ -9,7 +9,7 @@
 ;;;; provided with absolutely no warranty. See the COPYING and CREDITS
 ;;;; files for more information.
 
-(in-package "SB!VM")
+(in-package "SB-VM")
 
 ;;;; allocation helpers
 
@@ -32,7 +32,7 @@
   ;; unfortunately not enforced by PAD-DATA-BLOCK and
   ;; FIXED-ALLOC), so that ESP is always divisible by 8 (for
   ;; 32-bit lispobjs).  In that case, this AND instruction is
-  ;; unneccessary and could be removed.  If not, explain why.  -- CSR,
+  ;; unnecessary and could be removed.  If not, explain why.  -- CSR,
   ;; 2004-03-30
   (inst and esp-tn (lognot lowtag-mask))
   (inst lea alloc-tn (make-ea :byte :base esp-tn :disp lowtag))
@@ -62,34 +62,36 @@
 (defun allocation-inline (alloc-tn size)
   (let* ((ok (gen-label))
          (done (gen-label))
-         #!+(and sb-thread win32)
+         #+(and sb-thread win32)
          (scratch-tns (loop for my-tn in `(,eax-tn ,ebx-tn ,edx-tn ,ecx-tn)
                             when (and (not (location= alloc-tn my-tn))
                                       (or (not (tn-p size))
                                           (not (location= size my-tn))))
                             collect my-tn))
-         (tls-prefix #!+sb-thread :fs)
-         #!+(and sb-thread win32) (scratch-tn (pop scratch-tns))
-         #!+(and sb-thread win32) (swap-tn (pop scratch-tns))
+         (tls-prefix #+sb-thread :fs)
+         #+(and sb-thread win32) (scratch-tn (pop scratch-tns))
+         #+(and sb-thread win32) (swap-tn (pop scratch-tns))
          (free-pointer
            ;; thread->alloc_region.free_pointer
            (make-ea :dword
-                    :base (or #!+(and sb-thread win32)
+                    :base (or #+(and sb-thread win32)
                               scratch-tn)
                     :disp
-                    #!+sb-thread (* n-word-bytes thread-alloc-region-slot)
-                    #!-sb-thread (make-fixup "gc_alloc_region" :foreign)))
+                    #+sb-thread (* n-word-bytes thread-alloc-region-slot)
+                    ;; not a foreign-dataref because we don't support
+                    ;; dynamic core + no threads.
+                    #-sb-thread (make-fixup "gc_alloc_region" :foreign)))
          (end-addr
             ;; thread->alloc_region.end_addr
            (make-ea :dword
-                    :base (or #!+(and sb-thread win32)
+                    :base (or #+(and sb-thread win32)
                               scratch-tn)
                     :disp
-                    #!+sb-thread (* n-word-bytes (1+ thread-alloc-region-slot))
-                    #!-sb-thread (make-fixup "gc_alloc_region" :foreign 4))))
+                    #+sb-thread (* n-word-bytes (1+ thread-alloc-region-slot))
+                    #-sb-thread (make-fixup "gc_alloc_region" :foreign 4))))
     (unless (and (tn-p size) (location= alloc-tn size))
       (inst mov alloc-tn size))
-    #!+(and sb-thread win32)
+    #+(and sb-thread win32)
     (progn
       (inst push scratch-tn)
       (inst push swap-tn)
@@ -108,17 +110,17 @@
                  (#.esi-offset "alloc_overflow_esi")
                  (#.edi-offset "alloc_overflow_edi"))))
       (inst call (make-fixup dst :foreign)))
-    (inst jmp-short done)
+    (inst jmp done)
     (emit-label ok)
     ;; Swap ALLOC-TN and FREE-POINTER
     (cond ((and (tn-p size) (location= alloc-tn size))
            ;; XCHG is extremely slow, use the xor swap trick
-           #!-(and sb-thread win32)
+           #-(and sb-thread win32)
            (progn
              (inst xor alloc-tn free-pointer tls-prefix)
              (inst xor free-pointer alloc-tn tls-prefix)
              (inst xor alloc-tn free-pointer tls-prefix))
-           #!+(and sb-thread win32)
+           #+(and sb-thread win32)
            (progn
              (inst mov swap-tn free-pointer tls-prefix)
              (inst mov free-pointer alloc-tn tls-prefix)
@@ -128,7 +130,7 @@
            (inst mov free-pointer alloc-tn tls-prefix)
            (inst sub alloc-tn size)))
     (emit-label done)
-    #!+(and sb-thread win32)
+    #+(and sb-thread win32)
     (progn
       (inst pop swap-tn)
       (inst pop scratch-tn))
@@ -151,7 +153,7 @@
      (stack-allocation alloc-tn size lowtag))
     ;; Inline allocation can't work if (and (not sb-thread) sb-dynamic-core)
     ;; because boxed_region points to the linkage table, not the alloc region.
-    #!+(or sb-thread (not sb-dynamic-core))
+    #+(or sb-thread (not sb-dynamic-core))
     ((or (null node) (policy node (>= speed space)))
      (allocation-inline alloc-tn size))
     (t
@@ -163,11 +165,11 @@
 ;;; Allocate an other-pointer object of fixed SIZE with a single word
 ;;; header having the specified WIDETAG value. The result is placed in
 ;;; RESULT-TN.
-(defun fixed-alloc (result-tn widetag size node &optional stack-allocate-p)
-  (maybe-pseudo-atomic stack-allocate-p
+(defun alloc-other (result-tn widetag size node &optional stack-allocate-p)
+  (pseudo-atomic (:elide-if stack-allocate-p)
       (allocation result-tn (pad-data-block size) node stack-allocate-p
                   other-pointer-lowtag)
-      (storew (logior (ash (1- size) n-widetag-bits) widetag)
+      (storew (compute-object-header size widetag)
               result-tn 0 other-pointer-lowtag)))
 
 ;;;; CONS, LIST and LIST*
@@ -198,7 +200,7 @@
                      (storew reg ,list ,slot list-pointer-lowtag))))
              (let ((cons-cells (if star (1- num) num))
                    (stack-allocate-p (node-stack-allocate-p node)))
-               (maybe-pseudo-atomic stack-allocate-p
+               (pseudo-atomic (:elide-if stack-allocate-p)
                 (allocation res (* (pad-data-block cons-size) cons-cells) node
                             stack-allocate-p list-pointer-lowtag)
                 (move ptr res)
@@ -251,13 +253,13 @@
                                                    n-word-bytes))))
                    (inst and result (lognot lowtag-mask))
                    result))))
-      (pseudo-atomic
+      (pseudo-atomic ()
        (allocation result size node)
        (inst lea result (make-ea :byte :base result :disp other-pointer-lowtag))
        (sc-case type
          (immediate
-          (aver (typep (tn-value type) '(unsigned-byte 8)))
-          (storeb (tn-value type) result 0 other-pointer-lowtag))
+          (aver (typep (tn-value type) '(unsigned-byte 9)))
+          (store-widetag (tn-value type) result 0 other-pointer-lowtag))
          (t
           (storew type result 0 other-pointer-lowtag)))
        (sc-case length
@@ -265,7 +267,7 @@
           (let ((fixnum-length (fixnumize (tn-value length))))
             (typecase fixnum-length
               ((unsigned-byte 8)
-               (storeb fixnum-length result
+               (store-widetag fixnum-length result
                        vector-length-slot other-pointer-lowtag))
               (t
                (storew fixnum-length result
@@ -302,7 +304,7 @@
     (sc-case type
       (immediate
        (aver (typep (tn-value type) '(unsigned-byte 8)))
-       (storeb (tn-value type) result 0 other-pointer-lowtag))
+       (store-widetag (tn-value type) result 0 other-pointer-lowtag))
       (t
        (storew type result 0 other-pointer-lowtag)))
     (storew length result vector-length-slot other-pointer-lowtag)
@@ -318,7 +320,7 @@
   (:results (result :scs (descriptor-reg) :from :argument))
   (:node-var node)
   (:generator 37
-    (fixed-alloc result fdefn-widetag fdefn-size node)
+    (alloc-other result fdefn-widetag fdefn-size node)
     (storew name result fdefn-name-slot other-pointer-lowtag)
     (storew nil-value result fdefn-fun-slot other-pointer-lowtag)
     (storew (make-fixup 'undefined-tramp :assembly-routine)
@@ -332,20 +334,17 @@
   (:results (result :scs (descriptor-reg)))
   (:node-var node)
   (:generator 10
-   (maybe-pseudo-atomic stack-allocate-p
+   (pseudo-atomic (:elide-if stack-allocate-p)
      (let ((size (+ length closure-info-offset)))
        (allocation result (pad-data-block size) node
                    stack-allocate-p
                    fun-pointer-lowtag)
        (storew (logior (ash (1- size) n-widetag-bits) closure-widetag)
-               result 0 fun-pointer-lowtag))
-    ;; These two instructions are within the scope of PSEUDO-ATOMIC.
-    ;; This is due to scav_closure() assuming that it can always subtract
-    ;; FUN_RAW_ADDR_OFFSET from closure->fun to obtain a Lisp object,
-    ;; without any precheck for whether that word is currently 0.
-    (inst lea temp (make-ea-for-object-slot function simple-fun-code-offset
-                                            fun-pointer-lowtag))
-    (storew temp result closure-fun-slot fun-pointer-lowtag))))
+               result 0 fun-pointer-lowtag)))
+   ;; Done with pseudo-atomic
+   (inst lea temp (object-slot-ea function simple-fun-insts-offset
+                                           fun-pointer-lowtag))
+   (storew temp result closure-fun-slot fun-pointer-lowtag)))
 
 ;;; The compiler likes to be able to directly make value cells.
 (define-vop (make-value-cell)
@@ -354,7 +353,7 @@
   (:info stack-allocate-p)
   (:node-var node)
   (:generator 10
-    (fixed-alloc result value-cell-widetag value-cell-size node stack-allocate-p)
+    (alloc-other result value-cell-widetag value-cell-size node stack-allocate-p)
     (storew value result value-cell-value-slot other-pointer-lowtag)))
 
 ;;;; automatic allocators for primitive objects
@@ -369,7 +368,7 @@
   (:args)
   (:results (result :scs (any-reg)))
   (:generator 1
-    (inst lea result (make-fixup 'funcallable-instance-tramp :assembly-routine))))
+    (inst mov result (make-fixup 'funcallable-instance-tramp :assembly-routine))))
 
 (define-vop (fixed-alloc)
   (:args)
@@ -398,7 +397,7 @@
                                        ,@cases)))))
           (aver (null type))
           (inst call (make-fixup dst :assembly-routine)))
-        (maybe-pseudo-atomic stack-allocate-p
+        (pseudo-atomic (:elide-if stack-allocate-p)
          (allocation result (pad-data-block words) node stack-allocate-p lowtag)
          (when type
            (storew (logior (ash (1- words) n-widetag-bits) type)
@@ -409,7 +408,7 @@
 (define-vop (var-alloc)
   (:args (extra :scs (any-reg)))
   (:arg-types positive-fixnum)
-  (:info name words type lowtag)
+  (:info name words type lowtag stack-allocate-p)
   (:ignore name)
   (:results (result :scs (descriptor-reg) :from (:eval 1)))
   (:temporary (:sc any-reg :from :eval :to (:eval 1)) bytes)
@@ -423,7 +422,6 @@
     (inst lea header                    ; (w-1 << 8) | type
           (make-ea :dword :base header :disp (+ (ash -2 n-widetag-bits) type)))
     (inst and bytes (lognot lowtag-mask))
-    (pseudo-atomic
-     (allocation result bytes node)
-     (inst lea result (make-ea :byte :base result :disp lowtag))
+    (pseudo-atomic (:elide-if stack-allocate-p)
+     (allocation result bytes node stack-allocate-p lowtag)
      (storew header result 0 lowtag))))
